
import datetime
from vital_ai_vitalsigns_core.model.VitalServiceConfig import VitalServiceConfig


class VitalServiceIndexedDBConfig(VitalServiceConfig):
        graphQueries: str
        selectQueries: str

