
import datetime
from vital_ai_vitalsigns_core.model.DatabaseConnection import DatabaseConnection


class SparqlDatabaseConnection(DatabaseConnection):
        catalogName: str
        repositoryName: str

